package eGuideDog::Basic;

use strict;
use base qw(Exporter);
use LWP::Simple;
#use LWP::UserAgent;
use HTTP::Cookies;
use eGuideDog::I18N;
use I18N::LangTags::Detect;
use Cwd;

#use constant PAGE_SIZE => 10;

sub generate_tmpfile();
sub generate_url($);
sub debug($);
sub load_config();
sub gettext($);
sub get_full_path;
sub DESTROY;

#our $browser;
#our $speaker;
#our %index;
#our %config;
#our @history_links;
#our @history_speech;
our @EXPORT = qw(PAGE_SIZE
		 $browser
		 @history_links @history_speech
		 %index %config
		 gen_tmpfile generate_url gettext debug get_full_path);

my @languages = I18N::LangTags::Detect::detect();
my $I18N_handle = eGuideDog::I18N->get_handle(@languages); #|| die "what language?";

sub DESTROY {
#  $browser->cookie_jar->save();
}

# start HTTP user agent
#$browser = LWP::UserAgent->new();
#$browser->cookie_jar(HTTP::Cookies->new(file => 'cookies', autosave => 1));
#$browser->agent('eGuideDog');
#$browser->cookie_jar->load();

# setting
load_config();

# read config file
# the validity of the config file is not check, I hope this can be do in the future!!
#open(CONFIG_FILE, "config");
#my %config;
#my @pairs;
#while(<CONFIG_FILE>) {
#  chomp;
#  @pairs = split /=/;
#  $pairs[0] =~ s/^ //;
#  $pairs[0] =~ s/ $//;

#  $pairs[1] =~ s/^ //;
#  $pairs[1] =~ s/ $//;
#  next if ($pairs[0] eq "" or $pairs[1] eq "");
#  $config{$pairs[0]} = $pairs[1];
#}

# return config option
# getConfig(option)
sub getConfig($) {
  my $option = shift;
#  $config{$option};
}

sub gen_tmpfile {
  my $base = shift;

  # $< is $UID
  my $tmpdir = "/tmp/e-guidedog-$</";

  mkdir $tmpdir if (not -e $tmpdir);

  if (not defined $base) {
    $base = $tmpdir;
  } else {
    $base = $tmpdir . $base;
  }

  my $inc = 0;
  $inc++ while (-e "$base.$inc");
  return "$base.$inc";
}

sub generate_tmpfile() {
#  use Cwd;

  my $filename = '/tmp/' . time;
  if (-e $filename) {
    my $inc = 2;
    $inc++ while (-e "$filename-$inc");
    $filename = "$filename-$inc";
  }

#  mkdir(getcwd . '/tmp') if (!-d getcwd . '/tmp');

  return $filename;
}

sub generate_url($) {
  my %index;
  my $url = shift;
  if ($url !~ /^http:\/\// && $url !~ /^https:\/\//) { # relative url
    my $base = $index{url_base};
    my $protocol;
    my $site_base;
    if ($index{url_base} =~ /^http:\/\//) {
      $base =~ s/^http:\/\/([^\/]*)//;
      $site_base = 'http://' . $1;
      $protocol = 'http://';
    } else {
      $base =~ s/^https:\/\/([^\/]*)//;
      $site_base = 'https://' . $1;
      $protocol = 'https://';
    }
    $base =~ s/[^\/]*$//;

    if ($base eq '') {
      $base = $site_base . '/';
    } else {
      $base = $site_base;
    }

    if ($url =~ /^\//) { # website relative
      $url = $site_base . $url;
    } else { # directory relative
      $url = $base . $url;
    }
  }
  return $url;
}

sub debug($) {
  my $mesg = shift;

  open(FILE, '>>:utf8', 'debug.log');
  print FILE '[' . scalar(localtime(time)) . ']' . $mesg . "\n";
  close(FILE);
}

sub load_config() {
# config
# |-log_sound(whether log sound)
# |-speech
#   |-language(default language)
#   |-languages
#     |-italy
#       |- ...
#     |-english
#       |-voice(default voice)
#       |-@voices
#   |-style(default voice style)
#   |-@styles
#   |-scheme
# |-history_speech_size(max number of sentences store in the memory)
my %config;
$config{history_speech_size} = 10; # default 10 sentences.
$config{log_speech} = 1;
$config{key_help} = 1;
$config{speech} = \my %speech;
  $speech{languages} = \my %languages;
  $speech{language} = 'english'; # default language
    %languages = (english => \my %voice_en,
		  italy => \my %voice_it);
      %voice_en = (voices => \my @voices_en,
		   voice => 'kal_diphone');
        @voices_en = ('kal_diphone');
      %voice_it = (voices => \my @voices_it,
		   voice => 'lp_diphone');
        @voices_it = ('lp_diphone');
  $speech{styles} = \my @styles;
    @styles = ('male_slow', 'default', 'male_fast');
  $speech{style} = 'default'; # or 'male_slow', 'male_fast'
  # The following scheme function should thanks for BLINUX project.
  $speech{scheme} = qq(
        (defvar Styles '((default 140 22 1.0))
          "Available voice styles")

        (defvar style_default 'default
          "Default voice style")

        (defvar current_style 'none
          "Current voice style")

        (define (Style selected_style)
          "(Style DEFINED_STYLE)
           Sets the pitch, pitch variance, and speed of the current voice.
           Type 'Styles' for a list of defined styles."
          (let ((style (assoc selected_style Styles)))
            (if (not style)
                (set! style (assoc 'default Styles)))
            (let ((model_mean (cadr (assoc 'model_f0_mean  int_lr_params)))
                  (model_std  (cadr (assoc 'model_f0_std   int_lr_params)))
                  (new_mean (cadr style))
                  (new_std (cadr (cdr style)))
                  (new_stretch (cadr (cdr (cdr style)))))
              (set! int_lr_params
                    (list
                     (list 'target_f0_mean new_mean)
                     (list 'target_f0_std  new_std)
                     (list 'model_f0_mean  model_mean)
                     (list 'model_f0_std   model_std)))
              (Parameter.set 'Duration_Stretch new_stretch)
              (set! current_style (car style))
              (list (car style) new_mean new_std new_stretch)
              )
            )
          )

        (define (NewStyle style_name mean std stretch)
          "(NewStyle STYLE_NAME MEAN STD STRETCH)
           Defines a new style; MEAN and STD refer to pitch mean and variance,
           while STRETCH refers to inverse speed, 1.0 being the standard."
          (set! Styles (cons (list style_name mean std stretch) Styles)))

        (if (probe_file "/etc/festival.conf")
            (load "/etc/festival.conf"))

        (NewStyle 'male_slow       100 22 1.1 )
        (NewStyle 'male_fast       110 22 0.70)
);
}

# overwritten GNU gettext
# in fact, it's wrapper of maketext
# This can be detected by 'xgettext.pl'
sub gettext($) {
  my $text = shift;
  my $result = $I18N_handle->maketext($text); 
 if ($result) {
    return $result;
  } else {
    return $text;
  }
}

# get full path according to URL
sub get_full_path {
  my $url = shift;

  # try appending 'file://' or 'http://' and absolut path if need
  if (!($url =~ /^file:\/\//
	|| $url =~ /^http:\/\//
	|| $url =~ /^https:\/\//)) {
    if (-e $url) {
      # try 'file://' protocol
      my $u;
      if (!($url =~ /^\//)) {
	$u = getcwd . '/' . $url;
      } else {
	$u = $url
      }
      $url = 'file://' . $u;
    } else {
      $url = 'http://' . $url;
    }
  }
  return $url;
}

1;
